@@ -2,6 +2,7 @@ package interpreter
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"os"
 	"strings"
@@ -73,6 +74,9 @@ func (i *Interpreter) evalACPDeclaration(node *parser.ACPDeclaration) (Value, er
 // executeACPWithString creates a new ACP session with a user message and sends the prompt.
 // This is called when: "Hello" | ACPAgent
 func (i *Interpreter) executeACPWithString(message string, acpVal *ACPValue) (Value, error) {
+	prevEnv := i.env
+	defer func() { i.env = prevEnv }()
+
 	startTime := time.Now()
 
 	// Emit agent.start event
@@ -127,6 +131,9 @@ func (i *Interpreter) executeACPWithString(message string, acpVal *ACPValue) (Va
 // sendPromptToACPSession sends a prompt to an existing ACP session.
 // This is called when: session | "Follow up message"
 func (i *Interpreter) sendPromptToACPSession(session *ACPSessionValue, message string) (Value, error) {
+	prevEnv := i.env
+	defer func() { i.env = prevEnv }()
+
 	// Check if session is closed
 	if session.Closed {
 		return nil, fmt.Errorf("cannot send prompt to closed ACP session")
@@ -256,7 +263,22 @@ func (i *Interpreter) handleACPSessionUpdate(session *ACPSessionValue, update *a
 
 			var toolErr error
 			if status == "failed" {
-				toolErr = fmt.Errorf("tool call failed")
+				errMsg := output
+				// If no content output, try to extract from RawOutput
+				if errMsg == "" && update.Update.RawOutput != nil {
+					if rawStr, ok := update.Update.RawOutput.(string); ok {
+						errMsg = rawStr
+					} else {
+						if rawBytes, err := json.Marshal(update.Update.RawOutput); err == nil {
+							errMsg = string(rawBytes)
+						}
+					}
+				}
+				if errMsg != "" {
+					toolErr = fmt.Errorf("tool call failed: %s", errMsg)
+				} else {
+					toolErr = fmt.Errorf("tool call failed")
+				}
 			}
 
 			i.EmitEvent(EventAgentToolEnd, createACPToolCallContext(session.Agent, toolCallID, toolName, nil, &durationMs, &output, toolErr))
@@ -337,6 +359,7 @@ func (i *Interpreter) getOrCreateACPClient(acpVal *ACPValue) (*acp.Client, error
 		Env:         env,
 		Cwd:         cwd,
 		InitTimeout: 30 * time.Second,
+		Logger:      i.logger,
 	}
 
 	// Create and connect client using the factory
