@@ -294,6 +294,9 @@ func (i *Interpreter) SetEnv(name, value string) {
 
 	// Run the export command - ignore errors since this is best-effort
 	_ = i.runner.Run(context.Background(), prog)
+
+	// Keep os.Environ() in sync so subprocesses see the change
+	os.Setenv(name, value)
 }
 
 // UnsetEnv removes an environment variable by running an unset command through the runner
@@ -314,6 +317,9 @@ func (i *Interpreter) UnsetEnv(name string) {
 
 	// Run the unset command - ignore errors since this is best-effort
 	_ = i.runner.Run(context.Background(), prog)
+
+	// Keep os.Environ() in sync so subprocesses see the change
+	os.Unsetenv(name)
 }
 
 // EvalString parses and evaluates a source string in the interpreter.
@@ -409,6 +415,9 @@ func (i *Interpreter) GetEventHandlers(eventName string) []*ToolValue {
 // The return value from the chain (if any non-null value) can be used
 // to override default behavior. The interpretation is event-specific.
 func (i *Interpreter) EmitEvent(eventName string, ctx Value) Value {
+	prevEnv := i.env
+	defer func() { i.env = prevEnv }()
+
 	handlers := i.eventManager.GetHandlers(eventName)
 	if len(handlers) == 0 {
 		return nil
@@ -511,6 +520,11 @@ func (i *Interpreter) wrapError(err error, node parser.Node) error {
 		return err
 	}
 
+	// Don't wrap thrown errors — preserve the original thrown value
+	if _, isThrown := err.(*ThrownError); isThrown {
+		return err
+	}
+
 	// Get the current goroutine's call stack
 	callStack := i.callStacks.get()
 
